@@ -118,7 +118,7 @@ QString Evaluator::stringValue(const Item *item, const QString &name,
 static QStringList toStringList(const QScriptValue &scriptValue)
 {
     if (scriptValue.isString()) {
-        return QStringList(scriptValue.toString());
+        return {scriptValue.toString()};
     } else if (scriptValue.isArray()) {
         QStringList lst;
         int i = 0;
@@ -130,15 +130,24 @@ static QStringList toStringList(const QScriptValue &scriptValue)
         }
         return lst;
     }
-    return QStringList();
+    return {};
 }
 
 QStringList Evaluator::stringListValue(const Item *item, const QString &name, bool *propertyWasSet)
+{
+    const auto result = optionalStringListValue(item, name, propertyWasSet);
+    return result ? *result : QStringList();
+}
+
+std::optional<QStringList> Evaluator::optionalStringListValue(
+        const Item *item, const QString &name, bool *propertyWasSet)
 {
     QScriptValue v = property(item, name);
     handleEvaluationError(item, name, v);
     if (propertyWasSet)
         *propertyWasSet = isNonDefaultValue(item, name);
+    if (v.isUndefined())
+        return std::nullopt;
     return toStringList(v);
 }
 
@@ -163,7 +172,7 @@ QScriptValue Evaluator::scriptValue(const Item *item)
         return scriptValue;
     }
 
-    auto edata = new EvaluationData;
+    const auto edata = new EvaluationData;
     edata->evaluator = this;
     edata->item = item;
     edata->item->setObserver(this);
@@ -175,7 +184,7 @@ QScriptValue Evaluator::scriptValue(const Item *item)
 
 void Evaluator::onItemPropertyChanged(Item *item)
 {
-    EvaluationData *data = attachedPointer<EvaluationData>(m_scriptValueMap.value(item));
+    auto data = attachedPointer<EvaluationData>(m_scriptValueMap.value(item));
     if (data)
         data->valueCache.clear();
 }
